#!/usr/bin/python3
from asyncore import compact_traceback
import ctypes
from multiprocessing.sharedctypes import Value
from select import select
import subprocess
from ctypes import *
from enum import Enum
import socket
import psutil
import json
import threading
import re
import time
import datetime
from pydbus import SystemBus
import syslog
import os
from gi.repository import GLib, Gtk, Gio
import gi
gi.require_version('Gtk', '3.0')

from kysecKeyProcessControl import *

class processInfo:
    def __init__(self, pid):
        self.pid = pid
        self.status = "run"
        self.command = ""
        self.apath = ""

        commpath = "/proc/" + self.pid +"/comm"
        with open(commpath) as fp:
            self.command = fp.readline()
        
        absolutepath = "/proc/" + self.pid + "/exe"
        self.apath = os.readlink(absolutepath)

class processMonitor:
    """
    <node>
        <interface name='com.kylin.processMonitor.startend'>
            <method name='setProcess' >
                <arg type='s' direction='in' name='list'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='getProcess' >
                <arg type='as' direction='out' name='list'/>
            </method>

            <method name='getCommandbyPid' >
                <arg type='s' direction='in' name='pid' />
                <arg type='s' direction='out' name='command' />
            </method>
        </interface>
    </node>
    """

    def __init__(self):
        syslog.syslog(syslog.LOG_DEBUG, "processMonitor init")
        try:
            self.log = SystemBus().get("org.log.sys_transmit", "/org/log/sys_transmit")
        except Exception:
            syslog.syslog(syslog.LOG_DEBUG, "log service is error")
            thread_load_transmit = threading.Thread(target = self.load_transmit,args=())
            thread_load_transmit.setDaemon(True)
            thread_load_transmit.start()

        self.auditlogPath = "/var/log/audit/audit.log"
        #self.auditlogPath = "./test.log"
        self.officeExecList = []
        self.processList = []

        #开启
        logMonitorThread = threading.Thread(target=self.logMonitor, args=(self,))
        logMonitorThread.start()

        print("start processMonitor...")

    @property
    def get_host_name(self):
        """获取当前用户名"""
        json_user_list = json.dumps(psutil.users())
        user_list = json.loads(json_user_list)
        if len(user_list) < 1:
            return "root"
        return user_list[0][0]

    @property
    def get_host_time(self):
        """获取本机时间
        return: cur_time
        """
        cur_time = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
        return cur_time

    @property
    def get_host_ip(self):
        """
        查询本机ip地址
        :return: ip
        """
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect(('8.8.8.8', 80))
            ip = s.getsockname()[0]
        except Exception:
            return "unknown"
        return ip

    def load_transmit(self):
        count = 0
        while count <= 5:
            time.sleep(2)
            syslog.syslog(syslog.LOG_DEBUG, "reload transmit, count = {}".format(count))
            try:
                self.log = SystemBus().get("org.log.sys_transmit", "/org/log/sys_transmit")
                break
            except Exception:
                count = count + 1
                syslog.syslog(syslog.LOG_DEBUG, "log service is error")

    def log_transmit(self, name, message: str):
        time = self.get_host_time
        hostname = self.get_host_name
        ip = self.get_host_ip
        log_message = "'time':'{}','hostname':'{}','ip':'{}','name':'{}','lv':'info','message':'{}'".format(time, hostname, ip, name, message)
        syslog.syslog(syslog.LOG_DEBUG, "log_messgae = {}".format(log_message))
        try:
            ret = self.log.log_transmit('{' + log_message + '}')
            syslog.syslog(syslog.LOG_DEBUG, "sent log ret = {}".format(ret))
        except Exception:
            syslog.syslog(syslog.LOG_DEBUG, "send log message failed")

    def getProcess(self):
        return self.officeExecList

    def setProcess(self, list):
        tmplist =  self.officeExecList
        self.officeExecList = json.loads(list)
        syslog.syslog(syslog.LOG_DEBUG, "setProcess list = {}".format(self.officeExecList))

        controller = KeyProcessControl()
        ret = controller.kysec_process_enable()
        if ret != 0:
            syslog.syslog(syslog.LOG_DEBUG, "enable kysec process protect failed")
            return ret

        for path in tmplist:
            ret = controller.kysec_ppro_delete(path)
            if ret != 0:
                syslog.syslog(syslog.LOG_DEBUG, "delete kysec process {} failed.".format(path))
        
        for path in self.officeExecList:
            ret = controller.kysec_ppro_add(path)
            if ret != 0:
                syslog.syslog(syslog.LOG_DEBUG, "add {} failed".format(path))

        return ret

    def getCommandbyPid(self, pid) -> str:
        for pi in self.processList:
            if(pi.pid == pid):
                return pid.command

    @staticmethod
    def logMonitor(self):
        syslog.syslog(syslog.LOG_DEBUG, "logMonitor start")
        with open(self.auditlogPath) as f:
            f.seek(0,2)
            file_size = os.path.getsize(self.auditlogPath)
            while True:
                self.dailySend()
                new_file_size = os.path.getsize(self.auditlogPath)
                if new_file_size < file_size - 1000:
                    syslog.syslog(syslog.LOG_DEBUG, "reload audit.log")
                    f.seek(0,2)
                data = f.readlines()
                
                for line in data: 
                    if line == '\n':    #空行不处理
                        continue
                    self.parseLog(line)

                file_size = new_file_size
                time.sleep(1)
                # 处理文件截断问题

    def sendOfficeInfo(self, pi:processInfo):
        message = "status:" + pi.status + ", comm:" + pi.apath
        self.log_transmit("processmonitor", message)

    def dailySend(self):
        now = datetime.datetime.now()
        if now.hour == 0 and now.minute == 0 and now.second == 0:
            self.sendInfo("start")
        elif now.hour == 23 and now.minute == 59 and now.second == 59:
            self.sendInfo("end")
        
        tmppilist = []
        for pi in self.processList:
            if not os.path.exists("/proc/" + pi.pid):
                continue
            tmppilist.append(pi)
        self.processList = tmppilist

    def parseLog(self, line):
        if "{ run }" in line:
            p = 'type=.* msg=audit\(([0-9]{10}).*\).*?{ run } for msg="pid=(.*) comm=.*" loginuid=(.*) session=.*'
            info = re.findall(p, line)

            pi = processInfo(info[0][1], info[0][0])
            if pi.apath in self.officeExecList:
                self.sendOfficeInfo(pi)

            self.processList.append(pi)
            
        if "{ exit }" in line:
            p = 'type=.* msg=audit\(([0-9]{10}).*\).*?{ exit } for msg="pid=(.*) comm=.*'
            info = re.findall(p, line)
            
            pi.status = "exit"
            if pi.apath in self.officeExecList:
                self.sendOfficeInfo(pi)

if __name__ == "__main__":
    loop = GLib.MainLoop()
    bus = SystemBus()
    bus.publish("com.kylin.processMonitor", processMonitor())
    loop.run()
